@@ -16,18 +16,6 @@
 
 package org.uberfire.ext.editor.commons.client;
 
-import java.util.Arrays;
-import java.util.Collection;
-import java.util.HashSet;
-import java.util.Optional;
-import java.util.Set;
-import java.util.function.Consumer;
-import java.util.function.Supplier;
-
-import javax.enterprise.event.Event;
-import javax.enterprise.event.Observes;
-import javax.inject.Inject;
-
 import com.google.gwt.core.client.Scheduler;
 import com.google.gwt.user.client.ui.IsWidget;
 import elemental2.promise.Promise;
@@ -45,9 +33,10 @@ import org.uberfire.ext.editor.commons.client.event.ConcurrentDeleteAcceptedEven
 import org.uberfire.ext.editor.commons.client.event.ConcurrentDeleteIgnoredEvent;
 import org.uberfire.ext.editor.commons.client.event.ConcurrentRenameAcceptedEvent;
 import org.uberfire.ext.editor.commons.client.event.ConcurrentRenameIgnoredEvent;
+import org.uberfire.ext.editor.commons.client.file.popups.DeletePopUpPresenter;
 import org.uberfire.ext.editor.commons.client.history.VersionRecordManager;
 import org.uberfire.ext.editor.commons.client.menu.BasicFileMenuBuilder;
-import org.uberfire.ext.editor.commons.client.menu.DownloadMenuItem;
+import org.uberfire.ext.editor.commons.client.menu.DownloadMenuItemBuilder;
 import org.uberfire.ext.editor.commons.client.menu.MenuItems;
 import org.uberfire.ext.editor.commons.client.menu.common.SaveAndRenameCommandBuilder;
 import org.uberfire.ext.editor.commons.client.resources.i18n.CommonConstants;
@@ -58,6 +47,7 @@ import org.uberfire.ext.editor.commons.service.support.SupportsDelete;
 import org.uberfire.ext.editor.commons.service.support.SupportsRename;
 import org.uberfire.ext.editor.commons.service.support.SupportsSaveAndRename;
 import org.uberfire.ext.editor.commons.version.events.RestoreEvent;
+import org.uberfire.ext.widgets.common.client.common.ConcurrentChangePopup;
 import org.uberfire.java.nio.base.version.VersionRecord;
 import org.uberfire.mvp.Command;
 import org.uberfire.mvp.ParameterizedCommand;
@@ -66,16 +56,15 @@ import org.uberfire.workbench.events.NotificationEvent;
 import org.uberfire.workbench.model.menu.MenuItem;
 import org.uberfire.workbench.model.menu.Menus;
 
-import static org.uberfire.ext.editor.commons.client.menu.MenuItems.COPY;
-import static org.uberfire.ext.editor.commons.client.menu.MenuItems.DELETE;
-import static org.uberfire.ext.editor.commons.client.menu.MenuItems.DOWNLOAD;
-import static org.uberfire.ext.editor.commons.client.menu.MenuItems.HISTORY;
-import static org.uberfire.ext.editor.commons.client.menu.MenuItems.RENAME;
-import static org.uberfire.ext.editor.commons.client.menu.MenuItems.SAVE;
-import static org.uberfire.ext.editor.commons.client.menu.MenuItems.VALIDATE;
-import static org.uberfire.ext.widgets.common.client.common.ConcurrentChangePopup.newConcurrentDelete;
-import static org.uberfire.ext.widgets.common.client.common.ConcurrentChangePopup.newConcurrentRename;
-import static org.uberfire.ext.widgets.common.client.common.ConcurrentChangePopup.newConcurrentUpdate;
+import javax.enterprise.event.Event;
+import javax.enterprise.event.Observes;
+import javax.inject.Inject;
+import java.util.*;
+import java.util.function.Consumer;
+import java.util.function.Supplier;
+
+import static org.uberfire.ext.editor.commons.client.menu.MenuItems.*;
+import static org.uberfire.ext.widgets.common.client.common.ConcurrentChangePopup.*;
 
 public abstract class BaseEditor<T, M> {
 
@@ -126,11 +115,16 @@ public abstract class BaseEditor<T, M> {
     protected Event<ConcurrentRenameIgnoredEvent> concurrentRenameIgnoredEvent;
 
     @Inject
-    private DownloadMenuItem downloadMenuItem;
+    private DownloadMenuItemBuilder downloadMenuItemBuilder;
 
     @Inject
     protected Promises promises;
 
+    @Inject
+    protected DeletePopUpPresenter deletePopUpPresenter;
+
+    protected ConcurrentChangePopup concurrentChangePopup;
+
     protected Set<MenuItems> menuItems = new HashSet<>();
 
     protected PlaceRequest place;
@@ -270,7 +264,7 @@ public abstract class BaseEditor<T, M> {
     }
 
     protected MenuItem downloadMenuItem() {
-        return downloadMenuItem.build(getPathSupplier());
+        return downloadMenuItemBuilder.build(getPathSupplier());
     }
 
     Command getOnSave() {
@@ -288,10 +282,16 @@ public abstract class BaseEditor<T, M> {
                 .addContentSupplier(getContentSupplier())
                 .addIsDirtySupplier(isDirtySupplier())
                 .addSuccessCallback(onSuccess())
+                .addBeforeSaveAndRenameCommand(getBeforeSaveAndRenameCommand())
                 .build();
     }
 
-    Supplier<Boolean> getSaveValidator() {
+    protected Command getBeforeSaveAndRenameCommand() {
+        return () -> {
+        };
+    }
+
+    protected Supplier<Boolean> getSaveValidator() {
 
         return () -> {
 
@@ -388,33 +388,30 @@ public abstract class BaseEditor<T, M> {
             @Override
             public void execute(final ObservablePath.OnConcurrentUpdateEvent eventInfo) {
                 concurrentUpdateSessionInfo = eventInfo;
+                showConcurrentUpdatePopup();
             }
         });
 
-        path.onConcurrentRename(new ParameterizedCommand<ObservablePath.OnConcurrentRenameEvent>() {
-            @Override
-            public void execute(final ObservablePath.OnConcurrentRenameEvent info) {
-                newConcurrentRename(info.getSource(),
-                                    info.getTarget(),
-                                    info.getIdentity(),
-                                    onConcurrentRenameIgnoreCommand(path),
-                                    onConcurrentRenameCloseCommand(path)).show();
-            }
-        });
+        path.onConcurrentRename(this::onConcurrentRename);
 
-        path.onConcurrentDelete(new ParameterizedCommand<ObservablePath.OnConcurrentDelete>() {
-            @Override
-            public void execute(final ObservablePath.OnConcurrentDelete info) {
-                newConcurrentDelete(info.getPath(),
-                                    info.getIdentity(),
-                                    onConcurrentDeleteIgnoreCommand(path),
-                                    onConcurrentDeleteCloseCommand(path)).show();
-            }
-        });
+        path.onConcurrentDelete(this::onConcurrentDelete);
+    }
+
+    void onConcurrentRename(final ObservablePath.OnConcurrentRenameEvent info) {
+        baseView.hideBusyIndicator();
+        if (concurrentChangePopup == null) {
+            concurrentChangePopup = newConcurrentRename(info.getSource(),
+                                                        info.getTarget(),
+                                                        info.getIdentity(),
+                                                        onConcurrentRenameIgnoreCommand(path),
+                                                        onConcurrentRenameCloseCommand(path));
+        }
+        concurrentChangePopup.show();
     }
 
     Command onConcurrentRenameIgnoreCommand(final ObservablePath path) {
         return () -> {
+            concurrentChangePopup = null;
             disableMenus();
             concurrentRenameIgnoredEvent.fire(new ConcurrentRenameIgnoredEvent(path));
         };
@@ -422,25 +419,47 @@ public abstract class BaseEditor<T, M> {
 
     Command onConcurrentRenameCloseCommand(final ObservablePath path) {
         return () -> {
+            concurrentChangePopup = null;
             reload();
             concurrentRenameAcceptedEvent.fire(new ConcurrentRenameAcceptedEvent(path));
         };
     }
 
+    void onConcurrentDelete(final ObservablePath.OnConcurrentDelete info) {
+        baseView.hideBusyIndicator();
+        if (concurrentChangePopup == null) {
+            concurrentChangePopup = newConcurrentDelete(info.getPath(),
+                                                        info.getIdentity(),
+                                                        onConcurrentDeleteIgnoreCommand(path),
+                                                        onConcurrentDeleteCloseCommand(path));
+        }
+        concurrentChangePopup.show();
+    }
+
     Command onConcurrentDeleteIgnoreCommand(final ObservablePath path) {
         return () -> {
+            concurrentChangePopup = null;
             disableMenus();
+            disableDeletePopup();
             concurrentDeleteIgnoredEvent.fire(new ConcurrentDeleteIgnoredEvent(path));
         };
     }
 
     Command onConcurrentDeleteCloseCommand(final ObservablePath path) {
         return () -> {
+            concurrentChangePopup = null;
+            disableDeletePopup();
             placeManager.closePlace(place);
             concurrentDeleteAcceptedEvent.fire(new ConcurrentDeleteAcceptedEvent(path));
         };
     }
 
+    private void disableDeletePopup() {
+        if (deletePopUpPresenter.isOpened()) {
+            deletePopUpPresenter.cancel();
+        }
+    }
+
     private void onDelete() {
         Scheduler.get().scheduleDeferred(new Scheduler.ScheduledCommand() {
             @Override
