@@ -16,9 +16,15 @@
 
 package org.guvnor.common.services.project.client.security;
 
+import java.util.ArrayList;
 import java.util.Collection;
+import java.util.Collections;
+import java.util.List;
+import java.util.Map;
 import java.util.Optional;
 import java.util.function.Function;
+import java.util.stream.Collectors;
+
 import javax.enterprise.context.ApplicationScoped;
 import javax.inject.Inject;
 
@@ -29,6 +35,7 @@ import org.guvnor.structure.contributors.Contributor;
 import org.guvnor.structure.contributors.ContributorType;
 import org.guvnor.structure.organizationalunit.OrganizationalUnit;
 import org.guvnor.structure.organizationalunit.config.RolePermissions;
+import org.guvnor.structure.repositories.Branch;
 import org.guvnor.structure.repositories.Repository;
 import org.guvnor.structure.security.RepositoryAction;
 import org.jboss.errai.common.client.api.Caller;
@@ -78,6 +85,12 @@ public class ProjectController {
     }
 
     public Promise<Boolean> canUpdateProject(final WorkspaceProject workspaceProject) {
+        return canUpdateBranch(workspaceProject,
+                               workspaceProject.getBranch());
+    }
+
+    public Promise<Boolean> canUpdateBranch(final WorkspaceProject workspaceProject,
+                                            final Branch branch) {
         if (workspaceProject.getMainModule() == null) {
             return promises.resolve(false);
         }
@@ -89,7 +102,7 @@ public class ProjectController {
         }
 
         return checkBranchPermission(workspaceProject,
-                                     workspaceProject.getBranch().getName(),
+                                     branch.getName(),
                                      RolePermissions::canWrite);
     }
 
@@ -160,6 +173,11 @@ public class ProjectController {
                                project.getBranch().getName());
     }
 
+    public Promise<Boolean> canSubmitChangeRequest(final WorkspaceProject project) {
+        return canSubmitChangeRequest(project,
+                                      project.getBranch().getName());
+    }
+
     public Promise<Boolean> canDeleteBranch(final WorkspaceProject project,
                                             final String branch) {
         if (project.getBranch().getName().equals("master")) {
@@ -177,6 +195,23 @@ public class ProjectController {
                                      RolePermissions::canDelete);
     }
 
+    public Promise<Boolean> canSubmitChangeRequest(final WorkspaceProject project,
+                                                   final String branch) {
+        if (project.getBranch().getName().equals("master")) {
+            return promises.resolve(false);
+        }
+
+        if (authorizationManager.authorize(project.getRepository(),
+                                           RepositoryAction.UPDATE,
+                                           user)) {
+            return promises.resolve(true);
+        }
+
+        return checkBranchPermission(project,
+                                     branch,
+                                     RolePermissions::canRead);
+    }
+
     boolean userIsAtLeast(final ContributorType type,
                           final Collection<Contributor> contributors) {
         return contributors.stream().anyMatch(c -> c.getUsername().equals(user.getIdentifier())
@@ -206,16 +241,61 @@ public class ProjectController {
                                       return s.loadBranchPermissions(project.getSpace().getName(),
                                                                      project.getRepository().getIdentifier(),
                                                                      branch);
-                                  }).then(branchPermissions -> {
-            final Optional<Contributor> userContributor = getUserContributor(project.getRepository().getContributors());
-            if (userContributor.isPresent()) {
-                final RolePermissions rolePermissions = branchPermissions.getPermissionsByRole().get(userContributor.get().getType().name());
-                if (rolePermissions != null) {
-                    return promises.resolve(Optional.of(rolePermissions));
-                }
-            }
+                                  }).then(branchPermissions -> promises.resolve(getBranchPermissionsForUser(project, branchPermissions.getPermissionsByRole())));
+    }
 
-            return promises.resolve(Optional.empty());
-        });
+    public Optional<RolePermissions> getBranchPermissionsForUser(final WorkspaceProject project,
+                                                                 final Map<String, RolePermissions> permissionsByRole) {
+        final Optional<Contributor> userContributor = getUserContributor(project.getRepository().getContributors());
+        if (userContributor.isPresent()) {
+            final RolePermissions rolePermissions = permissionsByRole.get(userContributor.get().getType().name());
+            if (rolePermissions != null) {
+                return Optional.of(rolePermissions);
+            }
+        }
+
+        return Optional.empty();
+    }
+
+    public Promise<List<Branch>> getReadableBranches(final WorkspaceProject project) {
+        if (project.getMainModule() == null) {
+            return promises.resolve(Collections.emptyList());
+        }
+
+        if (authorizationManager.authorize(project.getRepository(),
+                                           RepositoryAction.READ,
+                                           user)) {
+            return promises.resolve(new ArrayList<>(project.getRepository().getBranches()));
+        }
+
+        return getBranchesWithPermission(project, RolePermissions::canRead);
+    }
+
+    public Promise<List<Branch>> getUpdatableBranches(final WorkspaceProject project) {
+        if (project.getMainModule() == null) {
+            return promises.resolve(Collections.emptyList());
+        }
+
+        if (authorizationManager.authorize(project.getRepository(),
+                                           RepositoryAction.UPDATE,
+                                           user)) {
+            return promises.resolve(new ArrayList<>(project.getRepository().getBranches()));
+        }
+
+        return getBranchesWithPermission(project, RolePermissions::canWrite);
+    }
+
+    private Promise<List<Branch>> getBranchesWithPermission(final WorkspaceProject project,
+                                                            final Function<RolePermissions, Boolean> rolePermissionsCheck) {
+
+        return promises.promisify(projectPermissionsService, service -> {
+            return service.loadBranchPermissions(project.getSpace().getName(),
+                                                 project.getRepository().getIdentifier(),
+                                                 project.getRepository().getBranches().stream().map(Branch::getName).collect(Collectors.toList()));
+        }).then(branchPermissions -> promises.resolve(project.getRepository().getBranches().stream().filter(branch -> {
+            final Optional<RolePermissions> branchPermissionsForUser = getBranchPermissionsForUser(project,
+                                                                                                   branchPermissions.get(branch.getName()).getPermissionsByRole());
+            return branchPermissionsForUser.map(rolePermissionsCheck).orElse(false);
+        }).collect(Collectors.toList())));
     }
 }
\ No newline at end of file
