@@ -25,6 +25,7 @@ import java.util.Map;
 import java.util.Set;
 import java.util.function.Predicate;
 import java.util.function.Supplier;
+
 import javax.annotation.PostConstruct;
 import javax.enterprise.context.ApplicationScoped;
 import javax.enterprise.event.Event;
@@ -56,6 +57,7 @@ import org.uberfire.client.workbench.events.PlaceGainFocusEvent;
 import org.uberfire.client.workbench.events.PlaceLostFocusEvent;
 import org.uberfire.client.workbench.events.SelectPlaceEvent;
 import org.uberfire.client.workbench.panels.impl.UnanchoredStaticWorkbenchPanelPresenter;
+import org.uberfire.experimental.service.auth.ExperimentalActivitiesAuthorizationManager;
 import org.uberfire.mvp.BiParameterizedCommand;
 import org.uberfire.mvp.Command;
 import org.uberfire.mvp.Commands;
@@ -81,8 +83,7 @@ import static org.uberfire.plugin.PluginUtil.toInteger;
 
 @SharedSingleton
 @EnabledByProperty(value = "uberfire.plugin.mode.active", negated = true)
-public class PlaceManagerImpl
-        implements PlaceManager {
+public class PlaceManagerImpl implements PlaceManager {
 
     /**
      * Activities that have been created by us but not destroyed (TODO: move this state tracking to ActivityManager!).
@@ -131,6 +132,14 @@ public class PlaceManagerImpl
     private WorkbenchLayout workbenchLayout;
     @Inject
     private LayoutSelection layoutSelection;
+    @Inject
+    private ExperimentalActivitiesAuthorizationManager activitiesAuthorizationManager;
+    @Inject
+    private AppFormerActivityLoader appFormerActivityLoader;
+
+    public interface AppFormerActivityLoader {
+        boolean triggerLoadOfMatchingEditors(final Path path, final Runnable callback);
+    }
 
     @PostConstruct
     public void initPlaceHistoryHandler() {
@@ -213,6 +222,13 @@ public class PlaceManagerImpl
                                                     UnanchoredStaticWorkbenchPanelPresenter.class.getName()));
     }
 
+    @Override
+    public void goTo(final String id,
+                     final HTMLElement addTo) {
+        final DefaultPlaceRequest place = new DefaultPlaceRequest(id);
+        goTo(place, addTo);
+    }
+
     @Override
     public void goTo(PlaceRequest place,
                      HTMLElement addTo) {
@@ -224,6 +240,16 @@ public class PlaceManagerImpl
                                                     UnanchoredStaticWorkbenchPanelPresenter.class.getName()));
     }
 
+    @Override
+    public void goTo(final PlaceRequest place,
+                     final elemental2.dom.HTMLElement addTo) {
+        closeOpenPlacesAt(panelsOfThisHTMLElement(addTo));
+
+        goToTargetPanel(place,
+                        panelManager.addCustomPanel(addTo,
+                                                    UnanchoredStaticWorkbenchPanelPresenter.class.getName()));
+    }
+
     private void closeOpenPlacesAt(Predicate<CustomPanelDefinition> filterPanels) {
         new HashSet<>(customPanels.values()).stream()
                 .filter(filterPanels)
@@ -239,6 +265,10 @@ public class PlaceManagerImpl
         return p -> p.getHasWidgetsContainer().isPresent() && p.getHasWidgetsContainer().get().equals(addTo);
     }
 
+    private Predicate<CustomPanelDefinition> panelsOfThisHTMLElement(elemental2.dom.HTMLElement addTo) {
+        return p -> p.getElemental2HtmlElementContainer().isPresent() && p.getElemental2HtmlElementContainer().get().equals(addTo);
+    }
+
     private void goToTargetPanel(final PlaceRequest place,
                                  final CustomPanelDefinition adoptedPanel) {
         if (existingWorkbenchActivities.containsKey(place)) {
@@ -261,6 +291,7 @@ public class PlaceManagerImpl
         if (place == null || place.equals(DefaultPlaceRequest.NOWHERE)) {
             return;
         }
+
         final ResolvedRequest resolved = resolveActivity(place);
 
         if (resolved.getActivity() != null) {
@@ -366,6 +397,7 @@ public class PlaceManagerImpl
      * {@code org.uberfire.client.mvp.PlaceManagerImpl.ignoreUnkownPlaces} property in {@link UberfirePreferences}.
      * @param place A non-null place request that could have originated from within application code, from within the
      * framework, or by parsing a hash fragment from a browser history event.
+     * @param lazyLoadingSuccessCallback
      * @return a non-null ResolvedRequest, where:
      * <ul>
      * <li>the Activity value is either the unambiguous resolved Activity instance, or null if the activity was
@@ -387,6 +419,10 @@ public class PlaceManagerImpl
             return existingDestination;
         }
 
+        if (appFormerActivityLoader.triggerLoadOfMatchingEditors(place.getPath(), () -> closeLazyLoadingScreenAndGoToPlace(place))) {
+            return new ResolvedRequest(null, new DefaultPlaceRequest("LazyLoadingScreen"));
+        }
+
         final Set<Activity> activities = activityManager.getActivities(resolvedPlaceRequest);
 
         if (activities == null || activities.size() == 0) {
@@ -419,6 +455,10 @@ public class PlaceManagerImpl
                                    resolvedPlaceRequest);
     }
 
+    private void closeLazyLoadingScreenAndGoToPlace(final PlaceRequest place) {
+        this.closePlace(new DefaultPlaceRequest("LazyLoadingScreen"), () -> this.goTo(place));
+    }
+
     private PlaceRequest resolvePlaceRequest(PlaceRequest place) {
         if (isaConditionalPlaceRequest(place)) {
             return resolveConditionalPlaceRequest((ConditionalPlaceRequest) place);
@@ -468,12 +508,15 @@ public class PlaceManagerImpl
         if (place == null) {
             return;
         }
+
         final ResolvedRequest resolved = resolveActivity(place);
 
         if (resolved.getActivity() != null) {
             final Activity activity = resolved.getActivity();
 
-            if (activity.isType(ActivityResourceType.EDITOR.name()) || activity.isType(ActivityResourceType.SCREEN.name())) {
+            if (activity.isType(ActivityResourceType.EDITOR.name()) ||
+                    activity.isType(ActivityResourceType.CLIENT_EDITOR.name()) ||
+                    activity.isType(ActivityResourceType.SCREEN.name())) {
                 final WorkbenchActivity workbenchActivity = (WorkbenchActivity) activity;
                 launchWorkbenchActivityInPanel(place,
                                                workbenchActivity,
@@ -849,26 +892,28 @@ public class PlaceManagerImpl
                                          titleDecoration,
                                          widget);
 
-        panelManager.addWorkbenchPart(place,
-                                      part,
-                                      panel,
-                                      activity.getMenus(),
-                                      uiPart,
-                                      activity.contextId(),
-                                      toInteger(panel.getWidthAsInt()),
-                                      toInteger(panel.getHeightAsInt()));
-        addSplashScreenFor(place);
+        activity.getMenus(menus -> {
+            panelManager.addWorkbenchPart(place,
+                                          part,
+                                          panel,
+                                          menus,
+                                          uiPart,
+                                          activity.contextId(),
+                                          toInteger(panel.getWidthAsInt()),
+                                          toInteger(panel.getHeightAsInt()));
+            addSplashScreenFor(place);
 
-        try {
-            activity.onOpen();
-            getPlaceHistoryHandler().registerOpen(activity,
-                                                  place);
-        } catch (Exception ex) {
-            lifecycleErrorHandler.handle(activity,
-                                         LifecyclePhase.OPEN,
-                                         ex);
-            closePlace(place);
-        }
+            try {
+                activity.onOpen();
+                getPlaceHistoryHandler().registerOpen(activity,
+                                                      place);
+            } catch (Exception ex) {
+                lifecycleErrorHandler.handle(activity,
+                                             LifecyclePhase.OPEN,
+                                             ex);
+                closePlace(place);
+            }
+        });
     }
 
     private IsWidget maybeWrapExternalWidget(WorkbenchActivity activity,
@@ -1013,11 +1058,12 @@ public class PlaceManagerImpl
      * process.
      */
     private void openPartsRecursively(PanelDefinition panel) {
+
         for (PartDefinition part : ensureIterable(panel.getParts())) {
+            activitiesAuthorizationManager.securePart(part, panel);
             final PlaceRequest place = part.getPlace().clone();
             part.setPlace(place);
-            goTo(part,
-                 panel);
+            goTo(part, panel);
         }
         for (PanelDefinition child : ensureIterable(panel.getChildren())) {
             openPartsRecursively(child);
@@ -1099,6 +1145,8 @@ public class PlaceManagerImpl
                 } else {
                     return;
                 }
+            } else {
+                activity.onClose();
             }
 
             getPlaceHistoryHandler().registerClose(activity,
