@@ -131,12 +131,24 @@ public class PluginPlaceManagerImpl implements PlaceManager {
         fail();
     }
 
+    @Override
+    public void goTo(String id,
+                     HTMLElement addTo) {
+        fail();
+    }
+
     @Override
     public void goTo(PlaceRequest place,
                      HTMLElement addTo) {
         fail();
     }
 
+    @Override
+    public void goTo(PlaceRequest place,
+                     elemental2.dom.HTMLElement addTo) {
+        fail();
+    }
+
     @Override
     public void goTo(final PartDefinition part,
                      final PanelDefinition panel) {
